##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

__version__='$Revision: 1.4 $'[11:-2]

import string
import rfc822

from Connection import Connection

from AccessControl.User import BasicUser
from cStringIO import StringIO


class imapUser(BasicUser):
    """ 
    The IMAP functionality is user-centric.  The whole Zope IMAP API
    is exposed through the imapUser object.  In Zope speak, an
    imapUser object works the same as any other type of user object.
    Typically it is used in DTML with the AUTHENTICATED_USER
    variable.  The simplest example of using an imapUser object is the 
    following DTML:

    <!--#var standard_html_header-->
      <!--#with AUTHENTICATED_USER-->
      <!--#tree expr="imapFolderTree(REQUEST)" sort=id nowrap=1-->
        <img src=<!--#var icon-->><!--#var id-->
      <!--#/tree-->
      <!--#/with-->
    <!--#var standard_html_footer-->

    This snipped will show you the tree breakdown of the user's IMAP
    folders.
    """
 
    def __init__(self, name, password, roles, domains):
        self.name   =name
        self.__     =password
        self.roles  =filter(None, map(string.strip, 
                                      string.split(roles, ',')))
        self.domains=filter(None, map(string.strip, 
                                      string.split(domains, ',')))
        self.conn = None

    def getUserName(self):
        return self.name

    def _getPassword(self):
        return self.__

    def getRoles(self):
        return self.roles
    
    def getDomains(self):
        return self.domains


    def connect(self, imap_server, imap_port):
        """ used by IMAPAdapter.IMAPAdapter.IMAPAdapter.validate """
        try:
            conn = Connection(imap_server, imap_port, self)
        except:
            return None
        self.conn = conn
        return conn


    # API

    #########  Tree methods

    def imapFolderTree(self, REQUEST=None):
	""" Tree layout of connection
        This method returns a live object tree that 
        can be rendered with the #tree tag.  
        """
	return self.conn


    #########  UID methods

    def imapGetUID(self, mbox, UID, REQUEST=None):
        """ Fetches one UID and returns it as an rfc822.Message
        object.  If the fetch raises an exception, returns an *empty*
        rfc822.Message object.  Probably should return None.
        """
        conn = self.conn.connection
        t, c = conn.select(mbox)
        try:
            typ, m = conn.uid('fetch', UID, 'RFC822')
            msg = rfc822.Message(StringIO(m[0][1]), 1)
        except:
            msg = rfc822.Message(StringIO(""), 1)
        conn.close()
        return msg


    def imapMoveUID(self, tobox, frombox, uid, REQUEST=None):
	""" Moves the message UID from frombox to tobox.

	"""
	conn = self.conn.connection
	conn.select(frombox)
        typ, msg = conn.fetch(uid, 'RFC822')
        typ, noth = conn.append(tobox, None, None, msg)
        conn.close()
	self.imapDeleteUID(frombox, messages, REQUEST)


    def imapDeleteUID(self, frombox, UID, REQUEST=None, action=None):
	""" Handles the 'Delete' button

	"""
	conn = self.conn.connection
	conn.select(frombox)
	#fetch each one and huck it
        typ, msg = conn.store(n, 'FLAGS', '(\Deleted)')
	conn.expunge()
	conn.close()


    #########  MailBox methods

    def imapCreateBox(self, newboxname, REQUEST=None):
	""" Create a new mailbox on the server

	"""
	conn = self.conn.connection
	conn.create(newboxname)


    def imapDeleteBox(self, box, REQUEST=None):
	""" Delete a mailbox on the server 

	"""
	conn = self.conn.connection
        conn.delete(box)

    
    def imapDeleteBoxes(self, boxes, REQUEST=None):
        """ Delete a sequence of boxes

        A little handier than doing it in DTML
        """
        conn = self.conn.connection
        for x in boxes:
            try:
                conn.delete(x)
            except:
                pass


    def imapSimpleSearchBox(self, keywords, box, REQUEST=None):
	""" use IMAP to search through messages for keywords

        This does only simple keyword searching through the message bodies.
	"""
	conn = self.conn.connection

	conn.select(box)
	typ, h = conn.uid('search', None, ('(BODY %s)' % keywords))
	conn.close()
        h = string.strip(h[0])
        if h:
            return string.split(h, ' ')
        return ()


    def imapSearchBox(self, search, box, REQUEST=None):
        """ do a full imap search.  
        The string 'search' should be a well formed IMAP search
        string.  See rfc2060 for the exact syntax.
        """
        conn = self.conn.connection

        conn.select(box)
        typ, h = conn.uid('search', None, search)
        h = string.strip(h[0])
        if h:
            return string.split(h, ' ')
        return ()


    def imapListBox(self, mbox, hits=None, REQUEST=None):
        """ lists mail uids in box
        """
        if hits is not None:
            return hits
        conn = self.conn.connection
        typ, c = conn.select(mbox)
        UIDs = string.split( (conn.uid("SEARCH", "ALL")[1][0]), ' ')
        conn.close()
        UIDs.reverse()
        return UIDs













